==> viha-5.15.tar.gz: viha-5.15/hlo-ok.php <==
<?php

# Copyright (C) 2009 Riku Kalinen
# License conditions in the file lisenssi.html
# Lisenssiehdot tiedostossa lisenssi.html

# sanity check functions for database additions, modifications and
# deletions.
# general format is ret = table_what_ok(&$msg,$id, ...)
# msg is error/warning message
# id is the main id of the record in question, new record => id = ''
# return codes:
#  0 (all ok, msg undefined)
#  1 (warnings in msg)
#  2 (fatal, reason in msg)

# hlo tarkastusfunktiot
# HUOM: ryhma_id vlitetn viittauksena koska sit voidaan joutua muuttamaan

function hlo_ok(&$msg,$id,$alue_id,$paivystys_id,$nimi,&$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id) {
  global $table;

  $msg = '';
  $ret = 0;
  
  if ($id != "") {
    $id_sql = "AND id != $id";
    $modify = 1;
    $tail = " Ei voi muuttaa.";
  } else {
    $modify = 0;
    $tail = " Ei voi list.";
  }

  $ret_p = permission_ids($msg,$alue_id,$paivystys_id);
  if ($ret_p == 2) {
    $msg .= $tail;
    return 2;
  } else {
    $ret = $ret_p;
  }

  # jos muutos, tarkastetaan ett on oikeus mys oleviin tietoihin
  if ($modify) {
    $w_alue_id = tab_col_id($table,"alue_id",$id);
    $w_paivystys_id = tab_col_id($table,"paivystys_id",$id);
    $ret_p = permission_ids($msg,$w_alue_id,$w_paivystys_id);
    if ($ret_p == 2) {
      $msg .= $tail;
      return 2;
    } else {
      $ret = $ret_p;
    }
  } # modify
  
  # tarpeelliset kentt
  if ($nimi == '') {
    $msg = 'Nimi on pakollinen tieto.';
    $msg .= $tail;
    return 2;
  }

  if (tab_col_id("ryhma","kutsu",$ryhma_id) == '') {
    $msg = "Ryhm $ryhma_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("hlo_tila","",$hlo_tila_id) == '') {
    $msg = "Henkiln tilakoodia $hlo_tila_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("hlo_asema","",$hlo_asema_id) == '') {
    $msg = "Henkiln asemakoodia $hlo_asema_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("jarjesto","",$jarjesto_id) == '') {
    $msg = "Jrjest $jarjesto_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  
  # duplikaatit
  $dup =
    ota_yksi("SELECT id
              FROM $table
              WHERE nimi = '$nimi'
              $id_sql");
  if ($dup != '') {
    $msg = "Tmn niminen henkil on jo olemassa (id=$dup).";
    $msg .= $tail;
    return 2;
  } 

  # jos ryhma ja paivystys on asetettu, niiden tytyy tsmt.
  if (($ryhma_id != '1') && ($paivystys_id != '1')) {
    if (tab_col_id("ryhma","paivystys_id",$ryhma_id) != $paivystys_id) {
      $msg = "Annetut ryhm($ryhma_id) ja operaatio($paivystys_id) eivt "
	. "sovi yhteen. Ryhm palautettu oletustilaan.";
      $ryhma_id = 1;
      $ret = 1;
    }
  }

  return $ret;
}

function hlo_add_ok(&$msg,$alue_id,$paivystys_id,$nimi,&$ryhma_id,$hlo_tila_id,
		    $hlo_asema_id,$jarjesto_id) {
  return hlo_ok($msg,'',$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id);
}

function hlo_save_ok(&$msg,$id,$alue_id,$paivystys_id,$nimi,&$ryhma_id,
		     $hlo_tila_id,$hlo_asema_id,$jarjesto_id) {
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id);
}

function hlo_set_ryhma_id_ok(&$msg,$id,$ryhma_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  # ryhma_id
  $hlo_tila_id = tab_col_id("hlo","hlo_tila_id",$id);
  $hlo_asema_id = tab_col_id("hlo","hlo_asema_id",$id);
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id);
}

function hlo_set_hlo_tila_id_ok(&$msg,$id,$hlo_tila_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  $ryhma_id = tab_col_id("hlo","ryhma_id",$id);
  # hlo_tila_id
  $hlo_asema_id = tab_col_id("hlo","hlo_asema_id",$id);
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id);
}

# here, we have no hlo.id .
# XXX some extra checks possible?
function hlo_set_hlo_tila_id_by_ryhma_id_ok(&$msg,$ryhma_id,$hlo_tila_id) {
  if (tab_col_id("ryhma","kutsu",$ryhma_id) == '') {
    $msg = "Ryhm $ryhma_id ei lydy.";
    return 2;
  }
  if (tab_col_id("hlo_tila","",$hlo_tila_id) == '') {
    $msg = "Henkiln tilakoodia $hlo_tila_id ei lydy.";
    return 2;
  }

  return 0;
}

function hlo_set_hlo_asema_id_ok(&$msg,$id,$hlo_asema_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  $ryhma_id = tab_col_id("hlo","ryhma_id",$id);
  $hlo_tila_id = tab_col_id("hlo","hlo_tila_id",$id);
  # hlo_asema_id
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id);
}

function hlo_remove_ok(&$msg, $id, $log = true) {
  global $table;
  
  $ret = 0;
  
  $ret_p = permission_tab($msg, $table, $id,$log);
  if ($ret_p == 2) {
    return 2;
  } else {
    $ret = $ret_p;
  }
  
  # jos viittauksia laitteissa tai tavarassa, ei voi poistaa
  $dup = ota_yksi("SELECT COUNT(*) FROM laite WHERE hlo_id = $id");
  if ($dup != 0) {
    $msg = "Henkiln on viel viittauksia $dup viestivlineess eik henkil
voida poistaa ennen kuin nm viittaukset on siivottu.";
    return 2;
  }
  $dup = ota_yksi("SELECT COUNT(*) FROM tavara WHERE hlo_id = $id");
  if ($dup != 0) {
    $msg = "Henkiln on viel viittauksia $dup tavarassa eik henkil
voida poistaa ennen kuin nm viittaukset on siivottu.";
    return 2;
  }
  
  return $ret;
}

# onko ok ottaa henkil thn operaatioon?

function hlo_take_to_paivystys_ok(&$msg,$id) {
  global $prefix;
    
  $p_id = ota_yksi("SELECT paivystys_id FROM hlo WHERE id = $id");
  if ($p_id == s('paivystys')) {
    $msg = "Et voi ottaa henkil joka kuuluu jo thn operaatioon.";
    return 2;
  }

  $hlo_alue_id = tab_col_id("hlo","alue_id",$id);
  if ($hlo_alue_id == 2)
    return 0;
  if ($hlo_alue_id == 1) {
    $msg = "Henkil (id=$id) ei kuulu millekn alueelle. Ei voi ottaa.";
    return 2;
  }
  $paivystys_alue_id = tab_col_id("paivystys","alue_id",
				  s('paivystys'));
  if ($paivystys_alue_id == 2)
    return 0;
  if ($hlo_alue_id != $paivystys_alue_id) {
    $msg = "Henkiln ja operaation alueet eivt tsm"
      . " ($hlo_alue_id,$paivystys_alue_id). Ei voi ottaa.";
    return 2;
  }
  
  return 0;
}

# onko ok vapauttaa henkil tst operaatiosta?

function hlo_release_from_paivystys_ok(&$msg,$id) {
  global $prefix;
    
  $p_id = ota_yksi("SELECT paivystys_id FROM hlo WHERE id = $id");
  if ($p_id != s('paivystys')) {
    $msg = "Et voi vapauttaa henkil joka ei kuulu thn operaatioon.";
    return 2;
  }

  return 0;
}

?>

<!-- RCS $Id: hlo-ok.php,v 1.22 2009/04/18 14:51:47 riku Exp $ -->

==> viha-5.27.tar.gz: viha-5.27/hlo-ok.php <==
<?php

# Copyright (C) 2009 Riku Kalinen
# License conditions in the file lisenssi.html
# Lisenssiehdot tiedostossa lisenssi.html

# sanity check functions for database additions, modifications and
# deletions.
# general format is ret = table_what_ok(&$msg,$id, ...)
# msg is error/warning message
# id is the main id of the record in question, new record => id = ''
# return codes:
#  0 (all ok, msg undefined)
#  1 (warnings in msg)
#  2 (fatal, reason in msg)

# hlo tarkastusfunktiot
# HUOM: ryhma_id vlitetn viittauksena koska sit voidaan joutua muuttamaan

function hlo_ok(&$msg,$id,$alue_id,$paivystys_id,$nimi,&$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,$osaaminen_ids) {
  $table = "hlo";

  $msg = '';
  $ret = 0;
  
  if ($id != "") {
    $id_sql = "AND id != $id";
    $modify = 1;
    $tail = " Ei voi muuttaa.";
  } else {
    $modify = 0;
    $tail = " Ei voi list.";
  }

  $ret_p = permission_ids($msg,$alue_id,$paivystys_id);
  if ($ret_p == 2) {
    $msg .= $tail;
    return 2;
  } else {
    $ret = $ret_p;
  }

  # jos muutos, tarkastetaan ett on oikeus mys oleviin tietoihin
  if ($modify) {
    $w_alue_id = tab_col_id($table,"alue_id",$id);
    $w_paivystys_id = tab_col_id($table,"paivystys_id",$id);
    $ret_p = permission_ids($msg,$w_alue_id,$w_paivystys_id);
    if ($ret_p == 2) {
      $msg .= $tail;
      return 2;
    } else {
      $ret = $ret_p;
    }
  } # modify
  
  # tarpeelliset kentt
  if ($nimi == '') {
    $msg = 'Nimi on pakollinen tieto.';
    $msg .= $tail;
    return 2;
  }

  if (tab_col_id("ryhma","kutsu",$ryhma_id) == '') {
    $msg = "Ryhm $ryhma_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("hlo_tila","",$hlo_tila_id) == '') {
    $msg = "Henkiln tilakoodia $hlo_tila_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("hlo_asema","",$hlo_asema_id) == '') {
    $msg = "Henkiln asemakoodia $hlo_asema_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("jarjesto","",$jarjesto_id) == '') {
    $msg = "Jrjest $jarjesto_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  
  # duplikaatit
  $dup =
    ota_yksi("SELECT id
              FROM $table
              WHERE nimi = '$nimi'
              $id_sql");
  if ($dup != '') {
    $msg = "Tmn niminen henkil on jo olemassa (id=$dup).";
    $msg .= $tail;
    return 2;
  } 

  # osaamiset
  foreach ($osaaminen_ids as $osaaminen_id) {
    if (tab_col_id("osaaminen","id",$osaaminen_id) == 0) {
      $msg = "Osaamisprofiilia $osaaminen_id ei lydy.";
      $msg .= $tail;
      return 2;
    }
  }
  
  # jos ryhma ja paivystys on asetettu, niiden tytyy tsmt.
  if (($ryhma_id != '1') && ($paivystys_id != '1')) {
    if (tab_col_id("ryhma","paivystys_id",$ryhma_id) != $paivystys_id) {
      $msg = "Annetut ryhm($ryhma_id) ja operaatio($paivystys_id) eivt "
	. "sovi yhteen. Ryhm palautettu oletustilaan.";
      $ryhma_id = 1;
      $ret = 1;
    }
  }

  return $ret;
}

function hlo_add_ok(&$msg,$alue_id,$paivystys_id,$nimi,&$ryhma_id,$hlo_tila_id,
		    $hlo_asema_id,$jarjesto_id,$osaaminen_ids) {
  return hlo_ok($msg,'',$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,$osaaminen_ids);
}

function hlo_save_ok(&$msg,$id,$alue_id,$paivystys_id,$nimi,&$ryhma_id,
		     $hlo_tila_id,$hlo_asema_id,$jarjesto_id,$osaaminen_ids) {
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,$osaaminen_ids);
}

function hlo_set_ryhma_id_ok(&$msg,$id,$ryhma_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  # ryhma_id
  $hlo_tila_id = tab_col_id("hlo","hlo_tila_id",$id);
  $hlo_asema_id = tab_col_id("hlo","hlo_asema_id",$id);
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,array());
}

function hlo_set_hlo_tila_id_ok(&$msg,$id,$hlo_tila_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  $ryhma_id = tab_col_id("hlo","ryhma_id",$id);
  # hlo_tila_id
  $hlo_asema_id = tab_col_id("hlo","hlo_asema_id",$id);
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,array());
}

# here, we have no hlo.id .
# XXX some extra checks possible?
function hlo_set_hlo_tila_id_by_ryhma_id_ok(&$msg,$ryhma_id,$hlo_tila_id) {
  if (tab_col_id("ryhma","kutsu",$ryhma_id) == '') {
    $msg = "Ryhm $ryhma_id ei lydy.";
    return 2;
  }
  if (tab_col_id("hlo_tila","",$hlo_tila_id) == '') {
    $msg = "Henkiln tilakoodia $hlo_tila_id ei lydy.";
    return 2;
  }

  return 0;
}

function hlo_set_hlo_asema_id_ok(&$msg,$id,$hlo_asema_id) {
  $alue_id = tab_col_id("hlo","alue_id",$id);
  $paivystys_id = tab_col_id("hlo","paivystys_id",$id);
  $nimi = tab_col_id("hlo","nimi",$id);
  $ryhma_id = tab_col_id("hlo","ryhma_id",$id);
  $hlo_tila_id = tab_col_id("hlo","hlo_tila_id",$id);
  # hlo_asema_id
  $jarjesto_id = tab_col_id("hlo","jarjesto_id",$id);
  return hlo_ok($msg,$id,$alue_id,$paivystys_id,$nimi,$ryhma_id,$hlo_tila_id,
		$hlo_asema_id,$jarjesto_id,array());
}

function hlo_remove_ok(&$msg, $id, $log = true) {
  global $table;
  
  $ret = 0;
  
  $ret_p = permission_tab($msg, $table, $id,$log);
  if ($ret_p == 2) {
    return 2;
  } else {
    $ret = $ret_p;
  }
  
  # jos viittauksia laitteissa tai tavarassa, ei voi poistaa
  $dup = ota_yksi("SELECT COUNT(*) FROM laite WHERE hlo_id = $id");
  if ($dup != 0) {
    $msg = "Henkiln on viel viittauksia $dup viestivlineess eik henkil
voida poistaa ennen kuin nm viittaukset on siivottu.";
    return 2;
  }
  $dup = ota_yksi("SELECT COUNT(*) FROM tavara WHERE hlo_id = $id");
  if ($dup != 0) {
    $msg = "Henkiln on viel viittauksia $dup tavarassa eik henkil
voida poistaa ennen kuin nm viittaukset on siivottu.";
    return 2;
  }
  
  return $ret;
}

# onko ok ottaa henkil thn operaatioon?

function hlo_take_to_paivystys_ok(&$msg,$id) {
  global $prefix;
    
  $p_id = ota_yksi("SELECT paivystys_id FROM hlo WHERE id = $id");
  if ($p_id == s('paivystys')) {
    $msg = "Et voi ottaa henkil joka kuuluu jo thn operaatioon.";
    return 2;
  }

  $hlo_alue_id = tab_col_id("hlo","alue_id",$id);
  if ($hlo_alue_id == 2)
    return 0;
  if ($hlo_alue_id == 1) {
    $msg = "Henkil (id=$id) ei kuulu millekn alueelle. Ei voi ottaa.";
    return 2;
  }
  $paivystys_alue_id = tab_col_id("paivystys","alue_id",
				  s('paivystys'));
  if ($paivystys_alue_id == 2)
    return 0;
  if ($hlo_alue_id != $paivystys_alue_id) {
    $msg = "Henkiln ja operaation alueet eivt tsm"
      . " ($hlo_alue_id,$paivystys_alue_id). Ei voi ottaa.";
    return 2;
  }
  
  return 0;
}

# onko ok vapauttaa henkil tst operaatiosta?

function hlo_release_from_paivystys_ok(&$msg,$id) {
  global $prefix;
    
  $p_id = ota_yksi("SELECT paivystys_id FROM hlo WHERE id = $id");
  if ($p_id != s('paivystys')) {
    $msg = "Et voi vapauttaa henkil joka ei kuulu thn operaatioon.";
    return 2;
  }

  return 0;
}

# tehdn valinta kaikista osaamisprofiileista tietylle henkillle.

function select_osaaminen_ids($hlo_id) {
  $koko = ota_yksi("SELECT COUNT(*)
                    FROM osaaminen");
  if ($koko > 5)
    $koko = 5;
  echo "<select multiple size=\"$koko\" name=\"osaaminen_ids[]\">\n";

  $sql =
    "SELECT id, osaaminen
     FROM osaaminen
     ORDER BY osaaminen";
  $result = my_mysql_query($sql);
  if ($result) {
    while ($data = mysql_fetch_row($result)) {
      $option_value = $data[0];
      $option_label = $data[1];
      echo "<option value=\"$option_value\"";
      if (ota_yksi("SELECT COUNT(*)
                    FROM osaaminen_hlo
                    WHERE hlo_id = $hlo_id
                          AND osaaminen_id = $option_value"))
	echo " selected";
      echo ">$option_label\n";
    }
  }
  
  echo "</select>\n";
}

# nytetn taulukossa olevat osaamiset

function list_osaaminen_ids($osaaminen_ids) {
  foreach ($osaaminen_ids as $id) {
    echo " " . tab_col_id("osaaminen","",$id);
  }
  echo "\n";
}

# tehdn osaaminen_ids lista tietylle henkillle.
function fetch_osaaminen_ids($id) {
  $ret = array();
  $sql = "SELECT osaaminen_id FROM osaaminen_hlo WHERE hlo_id = $id";
  $result = my_mysql_query($sql);
  if ($result) {
    while ($d = mysql_fetch_row($result)) {
      $ret[] = $d[0];
    }
  }
  return $ret;
}

# asetetaan henkiln osaaminen_ids lista kantaan.
function set_osaaminen_ids($now,$id,$osaaminen_ids) {
  $now = date('Y-m-d H:i:s');
  $sql = "DELETE FROM osaaminen_hlo WHERE hlo_id = $id;";
  my_mysql_query($sql,true,true);
  foreach ($osaaminen_ids as $osaaminen_id) {
    $sql =
      "INSERT INTO osaaminen_hlo
       SET hlo_id = $id,
           osaaminen_id = $osaaminen_id,
           ts = '$now'";
    my_mysql_query($sql,true,true);
  }
}

# otetaan/vapautetaan henkil operaatioon/operaatiosta.
# paluukoodit ja herja normaaliin tapaan.

function ota_hlo_paivystys(&$herja, $hlo_id, $paivystys_id) {
  
  if (hlo_take_to_paivystys_ok($herja,$hlo_id) == 2)
    return 2;

  ilmoita_jos_tarpeen($hlo_id);
  $now = date('Y-m-d H:i:s');
  # asetetaan ryhma ja tila vain jos henkil ei ennestn kuulu thn
  # operaatioon. tilaksi tulee "Paikalla".
  $sql =
    "UPDATE hlo
     SET ryhma_id = 1,
         hlo_tila_id = 3,
         ts = '$now'
     WHERE id = $hlo_id AND paivystys_id != $paivystys_id";
  $result = my_mysql_query ($sql,true,true);
  $sql =
    "UPDATE hlo
     SET paivystys_id = $paivystys_id,
         halyssa = 1,
         ts = '$now'
     WHERE id = $hlo_id";
  $result = my_mysql_query ($sql,true,true);
  $tmp_nimi = ota_nimi($hlo_id);
  taktinen_loki("Otettu henkil $tmp_nimi.");
  add_hlo_ajat($hlo_id);

  return 0;
}

function vapauta_hlo_paivystys(&$herja, $hlo_id, $paivystys_id) {
  if ( hlo_release_from_paivystys_ok($herja,$hlo_id) == 2)
    return 2;
  
  $now = date('Y-m-d H:i:s');
  $sql =
    "UPDATE hlo
     SET ryhma_id = 1, hlo_tila_id = 5, halyssa = 0, ts = '$now'
     WHERE id = $hlo_id";
  $result = my_mysql_query ($sql,true,true);
  add_hlo_ajat($hlo_id);
  $sql =
    "UPDATE hlo
     SET paivystys_id = 1, ts = '$now'
     WHERE id = $hlo_id";
  $result = my_mysql_query ($sql,true,true);
  add_hlo_ajat($hlo_id);
  $tmp_nimi = ota_nimi($hlo_id);
  taktinen_loki("Vapautettu henkil $tmp_nimi");

  return 0;
}

function vapauta_kaikki($jarjesto_id, $paivystys_id) {
  $sql =
    "SELECT id,nimi
     FROM hlo
     WHERE jarjesto_id = $jarjesto_id
           AND paivystys_id = " . s("paivystys") . "
           AND halyssa = 1
     ORDER BY nimi";
  $result = my_mysql_query($sql);
  if ($result) {
    while ($a = mysql_fetch_array($result)) {
      $ret = vapauta_hlo_paivystys($herja, $a['id'], s("paivystys"));
      if ($ret != 2) {
	$tmp_nimi = ota_nimi($a['id']);
	tehty("Vapautettu $tmp_nimi.");
      }
      p_herja($ret,$herja);
    }
  }
}
  
?>

<!-- RCS $Id: hlo-ok.php,v 1.27 2009/10/01 16:05:32 riku Exp $ -->
